# coding:utf-8
import sys
from tools_hjh import DBConn, HTTPTools
from tools_hjh import Log
from tools_hjh import ThreadPool
from tools_hjh import Tools
from bs4 import BeautifulSoup
from tools_hjh import ChromePool
import os

date = Tools.locatdate()
log = Log('U:/MyFiles/MyPy/log/woqukan/' + date + '.log')

host = 'http://www.woqukan.com'

chrome_path = r'U:\MyApps\CentBrowser\App\chrome.exe'
chromedriver_path = r'U:\MyApps\CentBrowser\chromedriver.exe'

try:
    sys_argv_1 = sys.argv[1]
except:
    sys_argv_1 = 'download'
try:
    sys_argv_2 = sys.argv[2]
except:
    sys_argv_2 = None
try:
    sys_argv_3 = sys.argv[3]
except:
    sys_argv_3 = 2
    

def main(): 
    db = DBConn('sqlite', db='U:/MyFiles/MyPy/data/woqukan.db')
    if sys_argv_1 == 'download':
        m_urls = None
        createDatabase(db, rebuild=False)
        chrome = ChromePool(sys_argv_3, chrome_path, chromedriver_path, is_display_picture=False, is_hidden=False)
        m_urls = get_main(HTTPTools, '/category/dongman.html', page_num='')  # page_num = '' 为最大值
        get_e(db, chrome, HTTPTools, m_urls=m_urls)
        print_need_download_url(db)
    else:
        rename(db, path=r'U:\Li')

    
def createDatabase(db, rebuild=False):
    if rebuild:
        db.run('drop table if exists t_m')
    
    # m_title,e_title 共同唯一
    # d_url相同，标题不同：d_url对应的m_title,e_title由最新的update_time决定
    # 标题相同，d_rul不同：大概率存在一个d_url过期失效，如果两个都有效，都下载，取名时候由于有源文件名后缀，不会冲突
    t_m = '''
        create table if not exists t_m(
            m_url text, 
            m_title text,
            e_url text,
            e_title text,
            d_url text,
            d_old_title text,
            d_new_title text,
            d_status text,
            c_主演 text,
            c_导演 text,
            c_类型 text,
            c_地区 text,
            c_年份 text,
            c_简介 text,
            update_time text
        )
    '''

    db.run(t_m)


# 循环主页，获取节目url列表返回
def get_main(request, firstPageUrl, page_num=''):
    
    def get_main_one(idx):
        if idx > 1:
            pageurl = host + firstPageUrl.replace('dongman.html', 'dongman_' + str(idx) + '.html')
        else:
            pageurl = host + firstPageUrl
        m_url = ''
        
        try:
            page = request.get(pageurl)
            bs = BeautifulSoup(page, features="lxml")
            lis = bs.find_all('li', class_='col-md-6 col-sm-4 col-xs-3')
            for li in lis:
                try:
                    m_url = li.find('a', class_='stui-vodlist__thumb lazyload')['href'].strip()
                    urls.append(m_url)
                except Exception as _:
                    print(_)
            
            log.info('get_main_one', str(idx), len(urls))
        except Exception as _:
            log.error('get_main_one', str(idx), len(urls), _)
            pass
    
    tp = ThreadPool(16)
    urls = []
    if page_num is None or len(page_num) == 0 or page_num == '':
        pageurl = host + firstPageUrl
        page = request.get(pageurl)
        bs = BeautifulSoup(page, features="lxml")
        href = bs.find('ul', class_='stui-page text-center cleafix').find_all('li')[-1].find('a')['href']
        page_num = int(href.split('_')[1].split('.html')[0])
    for idx in range(1, int(page_num) + 1):
        tp.run(get_main_one, (idx,))
    tp.wait()
    urls.sort(reverse=True)
    return urls

    
# 根据url，获取节目信息，如果是MP4类型的，获取每集下载链接，整体入库
def get_e(db, chrome, request, m_urls):

    def get_e_one(m_url, db, times=1):
        try:
            params = []
            status_code = request.get_status_code(host + m_url)
            page = request.get(host + m_url)
            # Tools.echo(page, 'test.txt', 'w')
            bs = BeautifulSoup(page, features="lxml")
            m_title = bs.find('div', class_='stui-content__detail').find('h1', class_='title').text.strip()
            c_主演 = bs.find('div', class_='stui-content__detail').find_all('p', class_='data')[0].text.replace('主演：', '').strip()
            c_导演 = bs.find('div', class_='stui-content__detail').find_all('p', class_='data')[1].text.replace('导演：', '').strip()
            if '/' in c_导演:
                c_导演 = c_导演.split('/')[0]
            c_类型 = bs.find('div', class_='stui-content__detail').find_all('p', class_='data')[2].find_all('a')[0].text.strip()
            c_地区 = bs.find('div', class_='stui-content__detail').find_all('p', class_='data')[2].find_all('a')[1].text.strip()
            c_年份 = bs.find('div', class_='stui-content__detail').find_all('p', class_='data')[2].find_all('a')[2].text.strip()
            c_简介 = bs.find('div', class_='stui-content__detail').find('p', class_='desc detail hidden-xs').find('span', class_='detail-content').text.strip()
            
            divs = bs.find_all('div', class_='stui-pannel stui-pannel-bg clearfix', id='playlist')
            for div in divs:
                m_type = div.find('h3', class_='title').text
                if m_type == 'MP4' or m_type == '云视频':
                    m_type = m_type.replace('云点播', 'm3u8').replace('MP4', 'mp4').replace('云视频', 'mp4')
                    lis = div.find('ul', class_='stui-content__playlist').find_all('li')
                    for li in lis:
                        e_title = li.find('a').text
                        e_url = li.find('a')['href']
                        if m_type == 'mp4':
                            # 根据标题去库里面查找是否存在一个下载地址，如果存在视为已拥有，不再处理
                            sql = "select count(1) from t_m t where t.m_title = ? and t.e_title = ?"
                            rs = int(db.run(sql, (m_title, e_title)).get_rows()[0][0])
                            if rs == 0:
                                d_url = get_download_url(e_url, chrome)
                                update_time = Tools.locatdate()
                                if d_url.endswith('.mp4'):
                                    d_old_title = d_url.split('/')[-1]
                                    d_new_title = m_title + '-' + e_title + '[' + c_导演 + ']' + '[' + c_年份 + '][' + d_old_title.replace('.mp4', '') + '].mp4'
                                    d_new_title = d_new_title.replace('/', ' ').replace('\\', ' ').replace(':', '：').replace('*', '·').replace('?', '？').replace('"', '\'').replace('<', '[').replace('>', ']').replace('|', ' ')
                                    d_new_title = d_new_title.replace('&amp;amp;', ' ')
                                    d_new_title = Tools.merge_spaces(d_new_title)
                                else:
                                    d_old_title = None
                                    d_new_title = None
                                d_status = '0'
                                param = (m_url, m_title, e_url, e_title, d_url, d_old_title, d_new_title, d_status, c_主演, c_导演, c_类型, c_地区, c_年份, c_简介, update_time, m_title, e_title, d_url)
                                params.append(param)
                
            insert_sql = '''
                insert into t_m(m_url,m_title,e_url,e_title,d_url,d_old_title,d_new_title,d_status,c_主演,c_导演,c_类型,c_地区,c_年份,c_简介,update_time)
                select ?,?,?,?,?,?,?,?,?,?,?,?,?,?,? where not exists(select 1 from t_m where m_title = ? and e_title = ? and d_url = ?)
            '''
            num = db.run(insert_sql, params, wait=True)
            m_urls.remove(m_url)
            log.info('get_e_one', m_url, num, len(params), len(m_urls), times)
        except Exception as _:
            if times <= 5:
                times = times + 1
                get_e_one(m_url, db, times)
            else:
                m_urls.remove(m_url)
                log.error('get_e_one', m_url, status_code, times, str(_))

    tp = ThreadPool(16)
    if m_urls is not None:
        for m_url in m_urls.copy():
            tp.run(get_e_one, (m_url, db))
                        
    # 如果没有urls传入，视为更新数据库已有节目信息
    else:
        m_urls = db.run("select distinct m_url from t_m order by cast(replace(replace(m_url,'/show/',''),'.html','') as integer) desc").get_rows()
        for m_url in m_urls:
            tp.run(get_e_one, (m_url[0], db))
    tp.wait()


def get_download_url(url, chrome):
    page = chrome.get(host + url)
    bs = BeautifulSoup(page, features="lxml")
    # Tools.echo(page, 'tmp.txt', 'w')
    script_ = bs.find('div', class_='stui-player__iframe').find('script')
    download_url = str(script_).split(';var now="')[1].split('";')[0]
    return download_url


def print_need_download_url(db):
    
    sql = '''
        select t.d_url from t_m t 
        where t.d_status = '0'
        and t.d_url like 'https://s1.xiahi.com/dm/%.mp4'
        
    '''
    try:
        rss = db.run(sql).get_rows()
        for rs in rss:
            print(rs[0])
    except:
        pass


def rename(db, path='U:/Li'):
    for file in os.listdir(path):
        try:
            old_path = path + '/' + file
            if file.endswith('].mp4'):
                d_name = file.split('][')[-1].replace(']', '')
            else:
                d_name = file
            sql = 'select d_new_title from t_m where d_old_title = ? order by update_time desc' 
            new_name = db.run(sql, (d_name,)).get_rows()[0][0]
            new_path = path + '/' + new_name
            os.rename(old_path, new_path)
            db.run("update t_m set d_status = '1' where d_old_title = ? and d_new_title = ?", (d_name, new_name))
            print('ok ' + file + ' -> ' + new_name)
        except Exception as _:
            pass


if __name__ == '__main__':
    main()
    



# coding:utf-8
from tools_hjh.ThreadPool import ThreadPool


class DBConn:
    """ 维护一个关系型数据库连接池，目前支持oracle，pgsql，mysql，sqlite；支持简单的sql执行 """
    
    ORACLE = 'oracle'
    PGSQL = 'pgsql'
    MYSQL = 'mysql'
    SQLITE = 'sqlite'

    def __init__(self, dbtype, host=None, port=None, db=None, username=None, password=None, poolsize=2, encoding='UTF-8', lib_dir=None):
        """ 初始化连接池
                如果是sqlite，db这个参数是要显示给入的，内存数据库 db=':memory:'
                如果是oracle，db给入的是sid或是servername都是可以的 """
                
        self.dbtype = dbtype
        self.host = host
        self.port = port
        self.db = db
        self.username = username
        self.password = password
        self.poolsize = poolsize
        self.encoding = encoding
        self.lib_dir = lib_dir
        
        self.runtp = ThreadPool(1, save_result=True)
        self.dbpool = None
        
        self.config = {
            'host':self.host,
            'port':self.port,
            'database':self.db,
            'user':self.username,
            'password':self.password,
            'maxconnections':self.poolsize,  # 最大连接数
            'blocking':True,  # 连接数达到最大时，新连接是否可阻塞
            'reset':False
        }
        
        if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
            from dbutils.pooled_db import PooledDB
        if self.dbtype == "pgsql":
            import psycopg2
            self.dbpool = PooledDB(psycopg2, **self.config)
        elif self.dbtype == "mysql":
            import pymysql
            self.dbpool = PooledDB(pymysql, **self.config)
        elif self.dbtype == "sqlite":
            if self.db is None:
                self.db = ':memory:'
            import sqlite3
            from dbutils.persistent_db import PersistentDB
            self.dbpool = PersistentDB(sqlite3, database=self.db)
        elif self.dbtype == "oracle":
            import cx_Oracle
            if lib_dir is not None:
                cx_Oracle.init_oracle_client(lib_dir=lib_dir)
            try:
                dsn = cx_Oracle.makedsn(host, port, service_name=self.db)
                self.dbpool = cx_Oracle.SessionPool(user=username,
                                                    password=password,
                                                    dsn=dsn,
                                                    max=poolsize,
                                                    increment=1,
                                                    encoding=encoding)
            except:
                dsn = cx_Oracle.makedsn(host, port, sid=self.db)
                self.dbpool = cx_Oracle.SessionPool(user=username,
                                                    password=password,
                                                    dsn=dsn,
                                                    max=poolsize,
                                                    increment=1,
                                                    encoding=encoding)
    
    def run_procedure(self, sql, param=None, auto_commit=None):
        # 替换占位符
        if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
            sql = sql.replace('?', '%s')
        elif self.dbtype == 'oracle':
            sql = sql.replace('?', ':1')            
        else:
            pass
                
        # 获取连接
        if self.dbtype == "oracle":
            conn = self.dbpool.acquire()
        else:
            conn = self.dbpool.connection()
            
        cur = conn.cursor()
                
        # 执行非SELECT语句
        sql = sql.strip()
        if type(param) == list:
            cur.executemany(sql, param)
        elif type(param) == tuple:
            cur.execute(sql, param)
        elif param is None:
            cur.execute(sql)
        if auto_commit: 
            if sql.lower().strip().startswith("update") or sql.lower().strip().startswith("delete") or sql.lower().strip().startswith("insert"):
                conn.commit()
        rownum = cur.rowcount
        rs = rownum
        cur.close()
        conn.close()
        return rs
    
    def __run(self, sqls, param=None, auto_commit=None):

        # 替换占位符
        if self.dbtype == 'pgsql' or self.dbtype == 'mysql':
            sqls = sqls.replace('?', '%s')
        elif self.dbtype == 'oracle':
            sqls = sqls.replace('?', ':1')            
        else:
            pass
        
        sql_list = []
        
        # sql只有一行
        if not '\n' in sqls:
            sql_list.append(sqls.rstrip(';'))
            
        # sql有多行
        else:
            # 去掉每行的首尾空格、换行，再去掉最后一个;,去掉--开头的行
            str2 = ''
            for line in sqls.split('\n'):
                line = line.strip()
                if not line.startswith('--') and len(line) > 0:
                    str2 = str2 + line + '\n'
            for sql in str2.split(';\n'):
                if sql is not None and sql != '' and len(sql) > 0:
                    sql_list.append(sql)
        
        # 获取连接
        if self.dbtype == "oracle":
            conn = self.dbpool.acquire()
        else:
            conn = self.dbpool.connection()
            
        cur = conn.cursor()
        
        for sql in sql_list:
            # 执行SELECT语句
            if sql.lower().strip().startswith("select") or (sql.lower().strip().startswith("with") and 'select' in sql.lower()):
                sql = sql.strip()
                if param is None:
                    cur.execute(sql)
                elif type(param) == tuple:
                    cur.execute(sql, param)
                rs = QueryResults(cur, conn)
                
            # 执行非SELECT语句
            elif not sql.lower().strip().startswith("select"):
                sql = sql.strip()
                if type(param) == list:
                    cur.executemany(sql, param)
                elif type(param) == tuple:
                    cur.execute(sql, param)
                elif param is None:
                    cur.execute(sql)
                if auto_commit: 
                    if sql.lower().strip().startswith("update") or sql.lower().strip().startswith("delete") or sql.lower().strip().startswith("insert"):
                        conn.commit()
                rownum = cur.rowcount
                rs = rownum
                cur.close()
                conn.close()
        # cur.close()
        # conn.close()
        return rs

    def run(self, sql, param=None, wait=False, auto_commit=True):
        """ 执行点什么
        sql中的占位符可以统一使用“?”
        wait为True则会等待当前正在执行的sql，有bug，暂不处理，自用规避"""
        if wait == True:
            tpnum = self.runtp.run(self.__run, (sql, param, auto_commit))
            self.runtp.wait()
            rs = self.runtp.result_map.pop(tpnum)
            return rs
        else:
            return self.__run(sql, param, auto_commit)
        
    def insert(self, table_name, rows, pk_cols=''):
        """ 往指定table_name中插入数据，rows是一个多个元组的列表，每个元组表示一组参数；或者是一个元组 """
        if type(rows) == list and len(rows) > 0:
            row = rows[0]
        elif type(rows) == tuple:
            row = rows
        elif len(rows) == 0:
            return 0
        
        sql2 = ''
        pk_num = 0
        if ',' not in pk_cols and pk_cols != '':
            pk_num = 1
            sql2 = pk_cols + ' = ?'
        elif ',' in pk_cols and len(pk_cols.split(',')) == 1:
            pk_num = 1
            sql2 = pk_cols.strip().strip(',') + ' = ?'
        elif ',' in pk_cols and len(pk_cols.split(',')) > 1:
            pk_num = len(pk_cols.split(','))
            for pk in pk_cols.split(','):
                sql2 = sql2 + pk + ' = ? and '
            sql2 = sql2.rstrip('and ')
            
        param_num = '?'
        for _ in range(len(row) - 1 - pk_num):
            param_num = param_num + ', ?'
            
        if self.dbtype == 'oracle' or self.dbtype == 'mysql':
            if len(pk_cols) == 0:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' from dual'
            else:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' from dual where not exists(select 1 from ' + table_name + ' where ' + sql2 + ')'
        else:
            if len(pk_cols) == 0:
                sql = 'insert into ' + table_name + ' select ' + param_num
            else:
                sql = 'insert into ' + table_name + ' select ' + param_num + ' where not exists(select 1 from ' + table_name + ' where ' + sql2 + ')'
        
        return self.run(sql, rows)
    
    def close(self):
        try:
            self.dbpool.close()
        except:
            pass
        finally:
            self.dbpool = None
    
    def __del__(self):
        self.close()
    
        
class QueryResults:

    def __init__(self, cur, conn):
        self.cur = cur
        self.conn = conn
        self.row = ''

    def get_cols(self):
        col = []
        for c in self.cur.description:
            col.append(c[0])
        return tuple(col)

    def get_rows(self, rownum='all'):
        if rownum == 'all' or rownum == True or rownum == False:
            rows = self.cur.fetchall()
        elif int(rownum) == 1:
            rows = self.cur.fetchone()
            if len(rows) == 0:
                self.close()
        else:
            rows = self.cur.fetchmany(int(rownum))
            if len(rows) == 0:
                self.close()
            
        rows_new = []
        for row in rows:
            row_new = []
            for cell in row:
                if 'LOB' not in str(type(cell)):
                    cell_new = cell
                else:
                    cell_new = cell.read()
                row_new.append(cell_new)
            rows_new.append(row_new)
            
        if rownum == 'all' or rownum == True or rownum == False:
            self.close()
        
        return rows_new
    
    def next(self):
        self.row = self.cur.fetchone()
        if self.row == None:
            self.close()
            return False
        else:
            return True

    def get_row(self):
        row_new = []
        for cell in self.row:
            if 'LOB' not in str(type(cell)):
                cell_new = cell
            else:
                cell_new = cell.read()
            row_new.append(cell_new)
        return row_new

    def close(self):
        try:
            self.cur.close()
        except:
            pass
        try:
            self.conn.close()
        except:
            pass
        
    def __del__(self):
        self.close()




# coding:utf-8

from tools_hjh import Tools

db = []
db.append(('risk_rule', '11.111.26.138', 'extjczx', 'dmpjczx', 'repjczxa-e', 'extdcuz', 'dmpdcuzj', 'repdcuzc', 'dpdb'))
db.append(('risk_flow', '11.111.26.138', 'extjcrb', 'dmpjcrb', 'repjcrba-e', 'extdcuz', 'dmpdcuzj', 'repdcuzc', 'dpdb'))
db.append(('risk_busdata', '11.111.26.138', 'extjcrb', 'dmpjcrb', 'repjcrba-e', 'extdcuz', 'dmpdcuzj', 'repdcuzc', 'dpdb'))
db.append(('afs', '11.111.28.111', 'extlwdsp', 'dmplwdsp', 'replwdsp', 'extdcuy', 'dmpdcuy', 'repdcuyb', 'bdpdb'))
db.append(('warn', '11.111.28.111', 'extlwdsp', 'dmplwdsp', 'replwdsp', 'extdcuy', 'dmpdcuy', 'repdcuyb', 'bdpdb'))
db.append(('pmml_user', '11.111.28.111', 'extlwdsp', 'dmplwdsp', 'replwdsp', 'extdcuy', 'dmpdcuy', 'repdcuyb', 'bdpdb'))
db.append(('cis', '11.111.27.126', 'ext-cis', 'dmp-cis', 'repcis', 'extdcis', 'dmpdcis', 'repdcis4', 'edzxdb'))
db.append(('psbcap', '11.111.26.5', 'ext-psb', 'dmppsbc', 'repsp6a-e', 'extdcsp', 'dmpdcsp', 'repdcspz', 'psbc'))
db.append(('pls', '11.111.25.9', 'ext-pls', 'dmp-pls', 'reppls', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'zyxfjr'))
db.append(('juphoon', '11.111.25.9', 'ext-pls', 'dmp-pls', 'reppls', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'zyxfjr'))
db.append(('upmp_data', '11.111.27.126', 'ext-ulcp', 'dmp-ulcp', 'repulcp', 'extdcuz', 'dmpdcuz', 'repdcuze', 'edzxdb'))
db.append(('collectsm', '11.111.28.227', 'ext-cs', 'dmp-cs', 'repcs1-8', 'extdcuz', 'dmpdcuz', 'repdcuzd', 'csmdb'))
db.append(('smartc', '11.111.6.135', 'extxygw', 'dmpxygwb', 'repxygw', 'extdcuz', 'dmpdcuz', 'repdcuzd', 'smartc'))
db.append(('robot', '22.223.7.196', 'extjqr', 'dmpjqr', 'repjqr', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'jqrdb'))
db.append(('itmusr', '22.223.7.196', 'extjqr', 'dmpjqr', 'repjqr', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'jqrdb'))
db.append(('ai_robot', '22.223.7.196', 'extjqr', 'dmpjqr', 'repjqr', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'jqrdb'))
db.append(('acct_facade', '11.111.28.107', 'ext-hs', 'dmp-hs', 'rephes1-16', 'extdcuz', 'dmpdcuzh', 'repdcuzf', 'hszxdb'))
db.append(('basc_info', '11.111.28.107', 'ext-hs', 'dmp-hs', 'rephes1-16', 'extdcuz', 'dmpdcuzh', 'repdcuzf', 'hszxdb'))
db.append(('common_service', '11.111.28.107', 'ext-hs', 'dmp-hs', 'rephesu', 'extdcuy', 'dmpdcuy', 'repdcuyc', 'hszxdb'))
db.append(('ycuser', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
# db.append(('ycuser', '22.223.7.196', 'extyy', 'dmpyy', 'repcall', 'extdcub', 'dmpdcub', 'repdcubd', 'yydb'))
db.append(('dmp', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx1-4', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('wcp', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('uasc_acct_own', '11.111.29.194', 'extzwt', 'dmpzwt', 'repzwta-p', 'extdczw', 'dmpdczw', 'repdczwc', 'zwzxdb'))
db.append(('bcp_fe', '11.111.29.194', 'extzwt', 'dmpzwt', 'repzwta-p', 'extdczw', 'dmpdczw', 'repdczwc3', 'zwzxdb'))
db.append(('zyxf', '11.111.6.72', 'ext-all', 'dpe-all', 'repdall', 'extdcua', 'dmpdcua', 'repdcuad', 'zyxfchnl'))
db.append(('qfcenter', '11.111.25.7', 'extums', 'dmpums', 'repums', 'extdcub', 'dmpdcub', 'repdcubd', 'zyxfjr'))
db.append(('fmsmid', '11.111.25.7', 'extums', 'dmpums', 'repums', 'extdcub', 'dmpdcub', 'repdcubd', 'zyxfjr'))
db.append(('reporter', '11.111.25.7', 'extrep', 'dmprep', 'reprep', 'extdcub', 'dmpdcub', 'repdcubd', 'zyxfjr'))
db.append(('ffzx', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('imop_security', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('imop_salesman', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('imop_base', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('dzxyxt', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('magicbox', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('upmp_credit', '11.111.27.126', 'ext-ulcp', 'dmp-ulcp', 'repulcp', 'extdcuz', 'dmpdcuz', 'repdcuze', 'edzxdb'))
db.append(('afms', '11.111.27.62', 'ext-fqz', 'dmp-fqz', 'repfqz', 'extdcub', 'dmpdcub', 'repdcubb', 'fqzdb'))
db.append(('uc_center', '11.111.28.117', 'ext-uc', 'dmp-uc', 'repyhzx1-8', 'extdcuy', 'dmpdcuy', 'repdcuye', 'yhzxdb'))
db.append(('apcusr', '11.111.28.117', 'ext-uc', 'dmp-uc', 'repyhzx1-8', 'extdcuy', 'dmpdcuy', 'repdcuye', 'yhzxdb'))
db.append(('ucqi', '11.111.28.112', 'extucq', 'dmpucq', 'repucq', 'extdcuz', 'dmpdcuz', 'repdcuze', 'bdpdb'))
db.append(('zycrm', '11.111.6.202', 'extzysho', 'dmpzysho', 'repzysho', 'extdcuz', 'dmpdcuz', 'repdcuze', 'immall'))
db.append(('imop_activity', '11.111.6.202', 'extffzx', 'dmpffzx', 'repffzx', 'extdcub', 'dmpdcub', 'repdcubd', 'immall'))
db.append(('activity', '11.111.6.72', 'ext-ac', 'dpe-ac', 'repqdac', 'extdcub', 'dmpdcub', 'repdcubd', 'zyxfchnl'))
db.append(('core_processing', '11.111.28.219', 'extcore', 'dmpcore', 'repcore1-4', 'extdczw', 'dmpdczw', 'repdczw1', 'xxdb'))
db.append(('lwdsp', '11.111.28.112', 'extlwdsp', 'dmplwdsp', 'replwdsp', 'extdcuy', 'dmpdcuy', 'repdcuyb', 'bdpdb'))
db.append(('ai_scene', '11.111.28.112', 'extlwdsp', 'dmplwdsp', 'replwdsp', 'extdcuy', 'dmpdcuy', 'repdcuyb', 'bdpdb'))
db.append(('ycmgr', '11.111.6.38', 'ext_nkf', 'dmp_nkf', 'repnkf', 'extdcuz', 'dmpdcuz', 'repdcuze', 'nkfdb'))
db.append(('zyxfnfs', '11.111.6.79', 'ext-tyxx', 'dpe-tyxx', 'reptyxxa-h', 'extdcuz', 'dmpdcuz', 'repdcuzn', 'kfdb'))
db.append(('mop', '11.111.6.72', 'ext-mop', 'dmp-mop', 'repdmop', 'extdcuy', 'dmpdcuy', 'repdcuyg', 'zyxfchnl'))
db.append(('ffkfqz', '11.111.6.72', 'extkfqz', 'dpekfqz', 'repkfqz', 'extdcuz', 'dmpdcuz', 'repdcuze', 'zyxfchnl'))
db.append(('ums', '11.111.25.9', 'extums', 'dmpums', 'repums', 'extdcuy', 'dmpdcuy', 'repdcuyc', 'zyxfjr'))
db.append(('ucpayusr', '11.111.28.211', 'ext-zf', 'dmp-zf', 'rezfucp', 'extdcua', 'dmpdcua', 'repdcuac', 'ucpaydb'))
db.append(('map', '11.111.6.79', 'ext-tyxx', 'dpe-tyxx', 'reptyxx', 'extdcuz', 'dmpdcuz', 'repdcuzn', 'kfdb'))
db.append(('fap', '11.111.25.9', 'extums', 'dmpums', 'repums', 'extdcuy', 'dmpdcuy', 'repdcuyc', 'zyxfjr'))
db.append(('ibsuser', '11.111.25.9', 'extums', 'dmpums', 'repums', 'extdcuz', 'dmpdcuz', 'repdcuze', 'zyxfjr'))
db.append(('limit_center', '11.111.27.81', 'ext-edzx', 'dmp-edzx', 'repedzxa-e', 'extdcub', 'dmpdcub', 'repdcubc', 'edzxdb'))
db.append(('i9_lab', '11.111.28.121', 'exti9', 'dmpi9', 'repi9', 'extdcuy', 'dmpdcuy', 'repdcuyc', 'fwbz'))
db.append(('crs', '11.111.28.121', 'extcrs', 'dmpcrs', 'repcrs', 'extdcuy', 'dmpdcuy', 'repdcuyc', 'fwbz'))
db.append(('settle', '11.111.25.9', 'extzy', 'dmpzy', 'repzy', 'extdcuy', 'dmpdcuy', 'repdcuyh', 'zyxfjr'))

link_sids = ['yhzxdb', 'ucpaydb', 'csmdb', 'psbc', 'zwzxdb', 'dpdb']
link_users = []
for conn in db:
    if conn[8] in link_sids and conn[0] not in link_users:
        link_users.append(conn[0])

exists_user = []
for u in db:
    exists_user.append(u[0])

date_str = Tools.locatdate().replace('-', '')

tables = '''
collectsm.TEMP_adepter_RECEIPT
'''


def main():
    
    script_str = ''
    
    # 从tables找出用户.表，用户
    table_list = []
    user_list = []
    for table in tables.lower().split('\n'):
        if ' ' in table:
            print('表中存在空格：' + table)
            return
        if '.' in table:
            table_list.append(table.strip())
            user = table.split('.')[0]
            if user not in user_list:
                user_list.append(user)
                
    # 校验映射关系是否全部存在
    all_exists = True
    for user in user_list:
        if user in exists_user:
            pass
        else:
            all_exists = False
            print(user + ':不存在')
    if not all_exists:
        exit
                
    # 业务库到ODS库
    for user in user_list:
        script_str = script_str + '----------------------------------------------------------------------------------------------------------------------------------\n\n'
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                
        if str(ogg_mess[1]).startswith('11.111'):
            beijin = True
        else:
            beijin = False
            
        if beijin:
            script_str = script_str + '-- ' + user + ' 相关需求表 业务库到ODS库' + '\n'
            script_str = script_str + 'ssh oracle@' + ogg_mess[1] + '\n\n'        
        else:
            script_str = script_str + '-- ' + user + ' 相关需求表 业务库到ODS库' + '\n'
            script_str = script_str + 'ssh ' + ogg_mess[1] + '\nsu - oracle\n\n'        
            
        # 查询业务库表大小
        script_str = script_str + '-- 查询业务库表大小' + '\n'
        get_table_size_sql = 'sqlplus / as sysdba\ncol table_name format a50;\n'
        for table in table_list:
            if user == table.split('.')[0]:
                get_table_size_sql = get_table_size_sql + "select '" + table + "' table_name, (select sum(t.bytes)/1024/1024 from dba_segments t where t.owner = '" + table.split('.')[0].upper() + "' and t.segment_name = '" + table.split('.')[1].upper() + "') size_m, (select count(1) from dba_tables t2 where t2.owner = '" + table.split('.')[0].upper() + "' and t2.table_name = '" + table.split('.')[1].upper() + "') tab_exists, (select count(1) from dba_indexes t3 where t3.table_owner = '" + table.split('.')[0].upper() + "' and t3.table_name = '" + table.split('.')[1].upper() + "' and t3.uniqueness = 'UNIQUE') unique_idx_num from dual union all\n"
        script_str = script_str + get_table_size_sql.rstrip('all\n').rstrip(' union') + ';\n\nexit\n\n'
        
        # 查询表是否已经配置OGG、或者被屏蔽、或者配置了*
        script_str = script_str + '-- 查询表是否已经配置OGG、或者被屏蔽、或者配置了*' + '\n'
        for table in table_list:
            if user == table.split('.')[0]:
                script_str = script_str + '/odc/ogg_pre_check.sh ' + table + ' $ORACLE_SID\n'
        script_str = script_str + '\n'
        
        for table in table_list:
            if user == table.split('.')[0]:
                script_str = script_str + 'grep -i \'' + table + '\' /odc/dirprm/*.prm\n'
        script_str = script_str + '\n'
        
        # 备份配置文件
        script_str = script_str + '-- 备份配置文件\n'
        if beijin:
            script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[2] + '.prm /odc/dirprm/bak/' + ogg_mess[2] + '.prm.' + date_str + '\n'
            script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[3] + '.prm /odc/dirprm/bak/' + ogg_mess[3] + '.prm.' + date_str + '\n\n'
        else:
            script_str = script_str + 'cp /odc/ogg/dirprm/' + ogg_mess[2] + '.prm /odc/ogg/dirprm/bak/' + ogg_mess[2] + '.prm.' + date_str + '\n'
            script_str = script_str + 'cp /odc/ogg/dirprm/' + ogg_mess[3] + '.prm /odc/ogg/dirprm/bak/' + ogg_mess[3] + '.prm.' + date_str + '\n\n'
            
        # 加trandata
        if beijin:
            script_str = script_str + '-- 加trandata\n/odc/ggsci\nview param ' + ogg_mess[2] + '\ndblogin\n'
        else:
            script_str = script_str + '-- 加trandata\n/odc/ogg/ggsci\nview param ' + ogg_mess[2] + '\ndblogin\n'
        add_trandata = ''
        for table in table_list:
            if user == table.split('.')[0]:
                add_trandata = add_trandata + "add trandata " + table + "\n"
        script_str = script_str + add_trandata + '\n'
        
        # 编辑ext
        script_str = script_str + '-- 编辑ext\nedit param ' + ogg_mess[2] + '\n'
        for table in table_list:
            if user == table.split('.')[0]:
                script_str = script_str + "include objname " + table + " &\n"
        script_str = script_str + '\n-- ' + date_str + '\n'
        for table in table_list:
            if user == table.split('.')[0]:
                script_str = script_str + "table " + table + " ;\n"
        
        # 编辑dmp
        script_str = script_str + '\n-- 编辑dmp\nedit param ' + ogg_mess[3] + '\n'
        script_str = script_str + '\n-- ' + date_str + '\n'
        for table in table_list:
            if user == table.split('.')[0]:
                script_str = script_str + "table " + table + " ;\n"
        
        # 重启ext、dmp
        script_str = script_str + '\n-- 重启ext、dmp\nstop ' + ogg_mess[2] + '\nstop ' + ogg_mess[3] + '\n\nstart ' + ogg_mess[2] + '\nstart ' + ogg_mess[3]
        script_str = script_str + '\n\nexit\n'
        
        # 到ODS库备份配置文件，停rep进程
        script_str = script_str + '\n-- 到ODS库备份配置文件，停rep进程\n'
        script_str = script_str + 'ssh oracle@11.111.24.149\n'
        if str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == 'a':
            rep_name = ogg_mess[4].split('a-')[0]
            begin_num = ord(str(ogg_mess[4])[-3])
            end_num = ord(str(ogg_mess[4])[-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'cp /odc/dirprm/' + rep_name + '' + chr(idx) + '.prm /odc/dirprm/bak/' + rep_name + '' + chr(idx) + '.prm.' + date_str + '\n'
            
            script_str = script_str + '\n/odc/ggsci\n'
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'stop ' + rep_name + '' + chr(idx) + '\n'
            
            script_str = script_str + '\nexit\n'
        elif str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(str(ogg_mess[4])[-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'cp /odc/dirprm/' + rep_name + '' + str(idx) + '.prm /odc/dirprm/bak/' + rep_name + '' + str(idx) + '.prm.' + date_str + '\n'
            
            script_str = script_str + '\n/odc/ggsci\n'
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'stop ' + rep_name + '' + str(idx) + '\n'
            
            script_str = script_str + '\nexit\n'
        elif str(ogg_mess[4])[-3] == '-':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-2]) * 10 + int(ogg_mess[4][-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'cp /odc/dirprm/' + rep_name + '' + str(idx) + '.prm /odc/dirprm/bak/' + rep_name + '' + str(idx) + '.prm.' + date_str + '\n'
            
            script_str = script_str + '\n/odc/ggsci\n'
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'stop ' + rep_name + '' + str(idx) + '\n'
            
            script_str = script_str + '\nexit\n'
        else:
            script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[4] + '.prm /odc/dirprm/bak/' + ogg_mess[4] + '.prm.' + date_str + '\n'
            script_str = script_str + '/odc/ggsci\nstop ' + ogg_mess[4] + '\n\nexit\n'
        
        # 业务库查询scn_number
        script_str = script_str + '\n-- 业务库查询scn_number\n'
        script_str = script_str + 'ssh oracle@' + ogg_mess[1] + '\nsqlplus / as sysdba\nselect to_char(current_scn) from v$database;\n-- scn_number_' + user + '_2ods\nexit\n'
        
        # 导出dmp文件
        script_str = script_str + '\n-- 导出dmp文件\n'
        table_str = ''
        for t in table_list:
            if t.split('.')[0] == user:
                table_str = table_str + t + ','
        table_str = table_str.strip(',')
        expdp = 'expdp \\"/ as sysdba \\" directory=dump dumpfile=expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_%U.dmp logfile=expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_.log tables=' + table_str + ' cluster=n parallel=8 compression=all flashback_scn=scn_number_' + user + '_2ods '
        script_str = script_str + expdp
        
        # scp到ODS库
        if beijin:
            script_str = script_str + '\n\n-- 传到ODS库 11.111.64.3上操作\n'
            script_str = script_str + 'scp oracle@' + ogg_mess[1] + ':/dbbak/dump/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp /tmp\n'
            script_str = script_str + 'scp /tmp/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp oracle@11.111.24.149:/dbbak/dump\n'
        else:
            script_str = script_str + '\n\n-- 想办法把文件传到ODS库 6.53操作\n'
            script_str = script_str + 'scp ' + ogg_mess[1] + ':/dbbak/dump/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp /tmp/\n'
            script_str = script_str + 'scp /tmp/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp 14.32.5.1:/tmp/\n'
            script_str = script_str + 'ssh 14.32.5.1\n'
            script_str = script_str + 'chown oracle:oinstall /tmp/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp\n'
            script_str = script_str + 'exit\n\n'
            script_str = script_str + '-- ods库操作\n'
            script_str = script_str + 'scp -P 7859 oracle@14.32.5.1:/tmp/expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_*.dmp /dbbak/dump\n'
        '''
        # ODS库编辑rep配置文件
        script_str = script_str + '\n-- ODS库编辑rep配置文件\nssh oracle@11.111.24.149\n/odc/ggsci\n'
        if str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == 'a':
            rep_name = ogg_mess[4].split('a-')[0]
            begin_num = ord(str(ogg_mess[4])[-3])
            end_num = ord(str(ogg_mess[4])[-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\nedit param ' + rep_name + '' + chr(idx) + '\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx - 96) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\n'
                script_str = script_str + editrep
        elif str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\nedit param ' + rep_name + '' + str(idx) + '\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\n'
                script_str = script_str + editrep
        elif str(ogg_mess[4])[-3] == '-' and str(ogg_mess[4])[-4] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-2]) * 10 + int(ogg_mess[4][-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\nedit param ' + rep_name + '' + str(idx) + '\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\n'
                script_str = script_str + editrep
        else:
            editrep = '\nedit param ' + ogg_mess[4] + '\n'
            editrep = editrep + '-- ' + date_str + '\n'
            map_ = ''
            for t in table_list:
                if user == t.split('.')[0]:
                    map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + ');\n'
            map_ = map_.strip('\n')
            editrep = editrep + map_    
            script_str = script_str + editrep
        '''    
        # ODS库编辑rep配置文件
        script_str = script_str + '\n-- ODS库编辑rep配置文件\nssh oracle@11.111.24.149\n/odc/ggsci\n'
        if str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == 'a':
            rep_name = ogg_mess[4].split('a-')[0]
            begin_num = ord(str(ogg_mess[4])[-3])
            end_num = ord(str(ogg_mess[4])[-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\necho \'\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx - 96) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\' >> /odc/dirprm/' + rep_name + chr(idx) + '.prm\n'
                script_str = script_str + editrep
        elif str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\necho \'\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\' >> /odc/dirprm/' + rep_name + str(idx) + '.prm\n'
                script_str = script_str + editrep
        elif str(ogg_mess[4])[-3] == '-' and str(ogg_mess[4])[-4] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-2]) * 10 + int(ogg_mess[4][-1]) + 1
            max_num = end_num - begin_num
            
            for idx in range(begin_num, end_num):
                editrep = '\necho \'\n'
                editrep = editrep + '-- ' + date_str + '\n'
                map_ = ''
                for t in table_list:
                    if user == t.split('.')[0]:
                        map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + '), filter(@range(' + str(idx) + ', ' + str(max_num) + '));\n'
                map_ = map_.strip('\n')
                editrep = editrep + map_ + '\' >> /odc/dirprm/' + rep_name + str(idx) + '.prm\n'
                script_str = script_str + editrep
        else:
            editrep = '\nedit param ' + ogg_mess[4] + '\n'
            editrep = editrep + '-- ' + date_str + '\n'
            map_ = ''
            for t in table_list:
                if user == t.split('.')[0]:
                    map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_' + user + '_2ods' + ');\n'
            map_ = map_.strip('\n')
            editrep = editrep + map_    
            script_str = script_str + editrep
            
        script_str = script_str + '\n\n-- 如果需要屏蔽表\n'
        mapexclude_ = '-- ' + date_str + '\n'
        for t in table_list:
            if user == t.split('.')[0]:
                mapexclude_ = mapexclude_ + 'mapexclude ' + t + ';\n'
        mapexclude_ = mapexclude_.strip('\n')
        script_str = script_str + mapexclude_ + '\n\n'
        
        tableexclude_ = '-- ' + date_str + '\n'
        for t in table_list:
            if user == t.split('.')[0]:
                tableexclude_ = tableexclude_ + 'tableexclude ' + t + ';\n'
        tableexclude_ = tableexclude_.strip('\n')
        script_str = script_str + tableexclude_
        
        script_str = script_str + '\n\nexit\n'
        
        # 查询目标库表情况
        script_str = script_str + '\n-- 查询目标库表情况' + '\n'
        get_table_size_sql = 'sqlplus / as sysdba\ncol table_name format a50;\n'
        for table in table_list:
            if user == table.split('.')[0]:
                get_table_size_sql = get_table_size_sql + "select '" + table + "' table_name, (select sum(t.bytes)/1024/1024 from dba_segments t where t.owner = '" + table.split('.')[0].upper() + "' and t.segment_name = '" + table.split('.')[1].upper() + "') size_m, (select count(1) from dba_tables t2 where t2.owner = '" + table.split('.')[0].upper() + "' and t2.table_name = '" + table.split('.')[1].upper() + "') tab_exists, (select count(1) from dba_indexes t3 where t3.table_owner = '" + table.split('.')[0].upper() + "' and t3.table_name = '" + table.split('.')[1].upper() + "' and t3.uniqueness = 'UNIQUE') unique_idx_num from dual union all\n"
        script_str = script_str + get_table_size_sql.rstrip('all\n').rstrip(' union') + ';\n\nexit\n\n'
        
        # 导入ODS库
        script_str = script_str + '\n-- 导入ODS库\n'
        script_str = script_str + 'impdp \\"/ as sysdba \\" directory=dump dumpfile=expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_%U.dmp logfile=expdp_2ogg_scn_number_' + user + '_2ods_' + date_str + '_.log cluster=n parallel=8\n'
        
        script_str = script_str + '\n-- 如果表空间不存在\n'
        script_str = script_str + "create tablespace XXX datafile '+DATAXX/' size 32760M autoextend off;\n"
        
        script_str = script_str + '\n-- 如果表用户不存在\n'
        script_str = script_str + "create user XXX identified by XXX default tablespace XXX;\n"
        script_str = script_str + "grant connect, resource to XXX;\n"
        
        # 启动rep进程
        script_str = script_str + '\n-- 启动rep进程\n/odc/ggsci\n'
        if str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == 'a':
            rep_name = ogg_mess[4].split('a-')[0]
            begin_num = ord(str(ogg_mess[4])[-3])
            end_num = ord(str(ogg_mess[4])[-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'start ' + rep_name + '' + chr(idx) + '\n'
            script_str = script_str + '\n'   
        elif str(ogg_mess[4])[-2] == '-' and str(ogg_mess[4])[-3] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'start ' + rep_name + '' + str(idx) + '\n'
            script_str = script_str + '\n'   
        elif str(ogg_mess[4])[-3] == '-' and str(ogg_mess[4])[-4] == '1':
            rep_name = ogg_mess[4].split('1-')[0]
            begin_num = 1
            end_num = int(ogg_mess[4][-2]) * 10 + int(ogg_mess[4][-1]) + 1
            
            for idx in range(begin_num, end_num):
                script_str = script_str + 'start ' + rep_name + '' + str(idx) + '\n'
            script_str = script_str + '\n'   
        else:
            startrep = 'start ' + ogg_mess[4] + '\n\nexit\n\n'
            script_str = script_str + startrep
    
    # ODS库到探查库
    script_str = script_str + '----------------------------------------------------------------------------------------------------------------------------------\n\n'
    script_str = script_str + '-- 相关需求表 ODS库到探查库' + '\n\n'
    
    script_str = script_str + '-- dblink部分' + '\n'
    script_str = script_str + 'ssh oracle@11.111.24.149\n'
    script_str = script_str + 'ssh -p 7859 oracle@14.32.5.1\n'
    script_str = script_str + 'sqlplus / as sysdba\n'
    for link_user in link_users:
        if link_user in user_list:
            user_list.remove(link_user)
            ogg_mess = ''
            for ogg_mess_ in db:
                if link_user == ogg_mess_[0]:
                    ogg_mess = ogg_mess_
            for table in table_list:
                if link_user == table.split('.')[0]:
                    if table.split('.')[1].upper() not in('NETAFS_REQUEST_COMPORE_INFO'):
                        table_list.remove(table)
                        # script_str = script_str + 'create synonym ' + table + ' for ' + table + '@' + ogg_mess[8] + ';\n'
                        script_str = script_str + 'create view ' + table + ' as select * from ' + table + '@dblk_' + ogg_mess[8] + ';\n'
    script_str = script_str + 'exit;\n\n'
    
    # 查询业务库表大小
    script_str = script_str + '-- 查询业务库表大小' + '\n'
    get_table_size_sql = 'sqlplus / as sysdba\ncol table_name format a50;\n'
    for table in table_list:
        get_table_size_sql = get_table_size_sql + "select '" + table + "' table_name, (select sum(t.bytes)/1024/1024 from dba_segments t where t.owner = '" + table.split('.')[0].upper() + "' and t.segment_name = '" + table.split('.')[1].upper() + "') size_m, (select count(1) from dba_tables t2 where t2.owner = '" + table.split('.')[0].upper() + "' and t2.table_name = '" + table.split('.')[1].upper() + "') tab_exists, (select count(1) from dba_indexes t3 where t3.table_owner = '" + table.split('.')[0].upper() + "' and t3.table_name = '" + table.split('.')[1].upper() + "' and t3.uniqueness = 'UNIQUE') unique_idx_num from dual union all\n"
    script_str = script_str + get_table_size_sql.rstrip('all\n').rstrip(' union') + ';\n\nexit\n\n'

    # 查询表是否已经配置OGG、或者被屏蔽、或者配置了*
    script_str = script_str + '-- 查询表是否已经配置OGG、或者被屏蔽、或者配置了*' + '\n'
    for table in table_list:
        script_str = script_str + '/odc/ogg_pre_check.sh ' + table + ' $ORACLE_SID\n'
    script_str = script_str + '\n'
    
    for table in table_list:
        script_str = script_str + 'grep -i \'' + table + '\' /odc/dirprm/*.prm\n'
    script_str = script_str + '\n'
    
    # 备份配置文件
    script_str = script_str + '-- 备份配置文件\n'
    o5 = []
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[5] not in o5:
                    script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[5] + '.prm /odc/dirprm/bak/' + ogg_mess[5] + '.prm.' + date_str + '\n'
                    script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[6] + '.prm /odc/dirprm/bak/' + ogg_mess[6] + '.prm.' + date_str + '\n'
                    o5.append(ogg_mess[5])
                        
    # 加trandata
    script_str = script_str + '\n-- 加trandata\n/odc/ggsci\nview param ' + ogg_mess[5] + '\ndblogin\n'
    add_trandata = ''
    for table in table_list:
        add_trandata = add_trandata + "add trandata " + table + "\n"
    script_str = script_str + add_trandata + '\n'
    
    # 编辑ext
    script_str = script_str + '-- 编辑ext\n'
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                script_str = script_str + '\nedit param ' + ogg_mess[5] + '\n'
                for table in table_list:
                    if user == table.split('.')[0]:
                        script_str = script_str + "include objname " + table + " &\n"
                script_str = script_str + '\n-- ' + date_str + '\n'
                for table in table_list:
                    if user == table.split('.')[0]:
                        script_str = script_str + "table " + table + " ;\n"
    
    # 编辑dmp
    script_str = script_str + '\n-- 编辑dmp'
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                script_str = script_str + '\nedit param ' + ogg_mess[6] + '\n'
                script_str = script_str + '\n-- ' + date_str + '\n'
                for table in table_list:
                    if user == table.split('.')[0]:
                        script_str = script_str + "table " + table + " ;\n"
                
    # 重启ext、dmp
    script_str = script_str + '\n-- 重启ext、dmp\n'
    o5 = []
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[5] not in o5:
                    script_str = script_str + 'stop ' + ogg_mess[5] + '\nstop ' + ogg_mess[6] + '\n'
                    o5.append(ogg_mess[5])
    script_str = script_str + '\n'
    o5 = []
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[5] not in o5:
                    script_str = script_str + 'start ' + ogg_mess[5] + '\nstart ' + ogg_mess[6] + '\n'
                    o5.append(ogg_mess[5])

    script_str = script_str + '\nexit\n'
                    
    # 到探查库备份配置文件，停rep进程
    script_str = script_str + '\n-- 到探查库备份配置文件，停rep进程\n'
    script_str = script_str + 'ssh -p 7859 oracle@14.32.5.1\n'
    
    o7 = []
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[7] not in o7:
                    if str(ogg_mess[7])[-2] == '-':
                        rep_name = ogg_mess[7].split('a-')[0]
                        begin_num = ord(str(ogg_mess[7])[-3])
                        end_num = ord(str(ogg_mess[7])[-1]) + 1
                        
                        for idx in range(begin_num, end_num):
                            script_str = script_str + 'cp /odc/dirprm/' + rep_name + '' + chr(idx) + '.prm /odc/dirprm/bak/' + rep_name + '' + chr(idx) + '.prm.' + date_str + '\n'
                    else:
                        script_str = script_str + 'cp /odc/dirprm/' + ogg_mess[7] + '.prm /odc/dirprm/bak/' + ogg_mess[7] + '.prm.' + date_str + '\n'
                    o7.append(ogg_mess[7])
                    
    script_str = script_str + '\n'     
    script_str = script_str + '\n/odc/ggsci\n'      
    o7 = []     
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[7] not in o7:
                    if str(ogg_mess[7])[-2] == '-': 
                        for idx in range(begin_num, end_num):
                            script_str = script_str + 'stop ' + rep_name + '' + chr(idx) + '\n'
                    else:
                        script_str = script_str + 'stop ' + ogg_mess[7] + '\n'
                    o7.append(ogg_mess[7])
                    
    script_str = script_str + '\nexit\n'

    # 业务库查询scn_number
    script_str = script_str + '\n-- ODS库查询scn_number\n'
    script_str = script_str + 'ssh oracle@11.111.24.149\nsqlplus / as sysdba\nselect to_char(current_scn) from v$database;\n-- scn_number_2tck\nexit\n'
    
    # 导出dmp文件
    script_str = script_str + '\n-- 导出dmp文件\n'
    table_str = ''
    for t in table_list:
        table_str = table_str + t + ','
    table_str = table_str.strip(',')
    expdp = 'expdp \\"/ as sysdba \\" directory=dump dumpfile=expdp_2tck_scn_number_2tck_' + date_str + '_%U.dmp logfile=expdp_2tck_scn_number_2tck_' + date_str + '_.log tables=' + table_str + ' cluster=n parallel=8 compression=all flashback_scn=scn_number_2tck '
    script_str = script_str + expdp
    
    # scp到探查库
    script_str = script_str + '\n\n-- 传到探查库\n'
    script_str = script_str + 'scp -P 7859 /dbbak/dump/expdp_2tck_scn_number_2tck_' + date_str + '_*.dmp oracle@14.32.5.1:/localdisk/dump\n'

    # 探查库编辑rep配置文件
    script_str = script_str + '\n-- 探查库编辑rep配置文件\nssh -p 7859 oracle@14.32.5.1\n/odc/ggsci'
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if str(ogg_mess[7])[-2] == '-':
                    rep_name = ogg_mess[7].split('a-')[0]
                    begin_num = ord(str(ogg_mess[7])[-3])
                    end_num = ord(str(ogg_mess[7])[-1]) + 1
                    max_num = end_num - begin_num
                    
                    for idx in range(begin_num, end_num):
                        editrep = '\n\nedit param ' + rep_name + '' + chr(idx) + '\n'
                        editrep = editrep + '-- ' + date_str + '\n'
                        map_ = ''
                        for t in table_list:
                            if user == t.split('.')[0]:
                                map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_2tck' + '), filter(@range(' + str(idx - 96) + ', ' + str(max_num) + '));\n'
                        map_ = map_.strip('\n')
                        editrep = editrep + map_ + '\n'
                        script_str = script_str + editrep
                        
                else:
                    editrep = '\n\nedit param ' + ogg_mess[7] + '\n'
                    editrep = editrep + '-- ' + date_str + '\n'
                    map_ = ''
                    for t in table_list:
                        if user == t.split('.')[0]:
                            map_ = map_ + 'map ' + t + ', target ' + t + ', filter(@getenv("transaction","csn")>scn_number_2tck' + ');\n'
                    map_ = map_.strip('\n')
                    editrep = editrep + map_    
                    script_str = script_str + editrep
    script_str = script_str + '\n\nexit\n'
        
    # 导入探查库
    script_str = script_str + '\n-- 导入探查库\n'
    script_str = script_str + 'impdp \\"/ as sysdba \\" directory=oradump dumpfile=expdp_2tck_scn_number_2tck_' + date_str + '_%U.dmp logfile=expdp_2tck_scn_number_2tck_' + date_str + '_.log cluster=n parallel=8 table_exists_action=replace\n'
    script_str = script_str + '\n-- 如果表空间不存在\n'
    script_str = script_str + "create tablespace XXX datafile '+DATA22/' size autoextend off;\n"
    
    # 启动rep进程
    script_str = script_str + '\n-- 启动rep进程\n/odc/ggsci\n'
    o7 = []
    for user in user_list:
        ogg_mess = ''
        for ogg_mess_ in db:
            if user == ogg_mess_[0]:
                ogg_mess = ogg_mess_
                if ogg_mess[7] not in o7:
                    if str(ogg_mess[7])[-2] == '-':
                        rep_name = ogg_mess[7].split('a-')[0]
                        begin_num = ord(str(ogg_mess[7])[-3])
                        end_num = ord(str(ogg_mess[7])[-1]) + 1
                        for idx in range(begin_num, end_num):
                            script_str = script_str + 'start ' + rep_name + '' + chr(idx) + '\n'
                        script_str = script_str + '\n'   
                    else:
                        startrep = 'start ' + ogg_mess[7] + '\n'
                        script_str = script_str + startrep
                    o7.append(ogg_mess[7])
    script_str = script_str + '\nexit\n'
        
    Tools.echo(script_str, 'ogg' + date_str + '.txt', 'w')
    print(script_str)

    
if __name__ == '__main__':
    main()
